﻿function New-SignedScript	{
  <#
      .SYNOPSIS
      Digitally signs a PowerShell script using a code signing certificate.

      .DESCRIPTION
      Digitally signs a PowerShell script using a code signing certificate installed on the local machine.

      .NOTES
      Version							  : 1.4 - See changelog at http://ucunleashed.com/2723
      Wish list						  :
      Rights Required			  : None
      Sched Task Required	  : No
      Lync/Skype4B Version  : N/A
      Author/Copyright      : © Pat Richard, Office Servers and Services (Skype for Business) MVP - All Rights Reserved
      Email/Blog/Twitter    : pat@innervation.com  https://ucunleashed.com  @patrichard
      Donations             : https://www.paypal.me/PatRichard
      Dedicated Post			  : https://ucunleashed.com/1496
      Disclaimer            : You running this script means you won't blame author(s) if this breaks your stuff. This script is
      provided AS IS without warranty of any kind. Author(s) disclaim all implied warranties including,
      without limitation, any implied warranties of merchantability or of fitness for a particular
      purpose. The entire risk arising out of the use or performance of the sample scripts and
      documentation remains with you. In no event shall author(s) be liable for any damages whatsoever
      (including, without limitation, damages for loss of business profits, business interruption,
      loss of business information, or other pecuniary loss) arising out of the use of or inability
      to use the script or documentation. Neither this script, nor any part of it other than those
      parts that are explicitly copied from others, may be republished without author(s) express written 
      permission.
      Acknowledgements 		  :
      Assumptions					  : ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
      Limitations					  :
      Known issues				  : None yet, but I'm sure you'll find some!

      .EXAMPLE
      New-SignedScript .\MyScript.ps1

      Description
      -----------
      Runs script with default values. If there is a single code signing certificate, it will be used. If there is more than one valid code signing certificate, a popup will display the certificates and allow one to be chosen.

      .EXAMPLE
      New-SignedScript .\MyScript.ps1 -CodeSigningCert 0

      Description
      -----------
      Signs the specified script, MyScript.ps1, with the first code signing certificate found.
  	
      .INPUTS
      None. You cannot pipe objects to this script.
  #>

  [CmdletBinding(SupportsShouldProcess, HelpUri = 'https://ucunleashed.com/1496')]
  param (
    # Path (including file name) of script to sign
    [parameter(ValueFromPipeline, ValueFromPipelineByPropertyName, Mandatory, HelpMessage = 'No filename specified')]
    [ValidatePattern('.ps1$|.psm1$')]
    [ValidateNotNullOrEmpty()]
    [string] $FileName,
		
    # Index number of code signing certificate to use
    [parameter(ValueFromPipeline, ValueFromPipelineByPropertyName)]
    [string] $CodeSigningCerificate
  ) # end begin block
  BEGIN	{
    Write-Verbose -Message 'Getting certificate available for codesigning'
    # [int] $totalvalidcerts = (Get-ChildItem Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.NotAfter -gt (Get-Date)}).length
    [int] $totalvalidcerts = (Get-ChildItem -Path Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.Verify()}).length
    if ($totalvalidcerts -eq 1){
      # [Security.Cryptography.X509Certificates.X509Certificate2] $cert = (Get-ChildItem Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.NotAfter -ge (Get-Date)})[0]
      [Security.Cryptography.X509Certificates.X509Certificate2] $cert = (Get-ChildItem -Path Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.Verify()})[0]
    }
    elseif (($CodeSigningCerificate) -and ($totalvalidcerts -gt 1)){
      # [Security.Cryptography.X509Certificates.X509Certificate2] $cert = (Get-ChildItem Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.NotAfter -ge (Get-Date)})[$CodeSigningCerificate]
      [Security.Cryptography.X509Certificates.X509Certificate2] $cert = (Get-ChildItem -Path Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.Verify()})[$CodeSigningCerificate]
    }else{
      # [Security.Cryptography.X509Certificates.X509Certificate2] $cert = (Get-ChildItem Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.NotAfter -ge (Get-Date)} | Out-GridView -Title "Valid certificates" -OutputMode Single)[0]
      [Security.Cryptography.X509Certificates.X509Certificate2] $cert = (Get-ChildItem -Path Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.Verify()} | Out-GridView -Title 'Valid certificates' -OutputMode Single)[0]
    }
  }
  PROCESS	{
    if ($cert){
      Write-Verbose -Message ('{0} valid certificate(s) found' -f $totalvalidcerts)
      Write-Verbose -Message ('Signing "{0}" with {1}' -f $FileName, $cert)
      $null = Set-AuthenticodeSignature -FilePath $FileName -Certificate $cert -TimestampServer 'http://timestamp.digicert.com'
      Write-Verbose -Message 'Checking validity'
      $valid = ((Get-AuthenticodeSignature -FilePath $FileName).status -Match 'Valid')
      if ($valid){
        Write-Verbose -Message ('Signing {0} succeeded' -f $FileName)
      }else{
        Write-Error -Message ('Signing {0} failed' -f $FileName)
      }
    }else{
      Write-Error -Message 'No valid codesigning certificate found'
    }		
  } # end process block
  END	{
    Remove-Variable -Name Valid
    Remove-Variable -Name FileName
    Remove-Variable -Name Cert
  } # end end block
} # end function New-SignedScript